@@ -1,10 +1,10 @@
 import sys
 import unittest
 
-import gpgme.tests
+import tests
 
 def test_suite():
-    return gpgme.tests.test_suite()
+    return tests.test_suite()
 
 if __name__ == '__main__':
     unittest.main(defaultTest='test_suite')
